package com.sd.business.controller.board;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sd.business.entity.board.dto.InOutStorageBoardSelectDto;
import com.sd.business.entity.board.dto.SalesBoardSelectDto;
import com.sd.business.entity.board.vo.HazardousInventoryVo;
import com.sd.business.entity.board.vo.InOutStorageBoardVo;
import com.sd.business.entity.board.vo.InTransitBomVo;
import com.sd.business.entity.board.vo.InventoryTurnoverRateVo;
import com.sd.business.service.board.InventoryBoardService;
import com.sd.framework.util.excel.util.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;


/**
 * <p>
 * 库存看板 前端控制器
 * </p>
 *
 * @author
 * @since 2023-11-16
 */
@RestController
@RequestMapping("/inventoryBoard")
public class InventoryBoardController {

    @Autowired
    private InventoryBoardService inventoryBoardService;

    /**
     * 获取bom危险库存列表分页
     * @return
     */
    @PostMapping("/getHazardousInventoryPage")
    public Page<HazardousInventoryVo> getHazardousInventoryPage(@RequestBody SalesBoardSelectDto dto) {
        return inventoryBoardService.getHazardousInventoryPage(dto);
    }

    /**
     * bom危险库存列表导出
     * @return
     */
    @PostMapping("/hazardousInventoryExportExcel")
    public void hazardousInventoryExportExcel(HttpServletResponse response, @RequestBody SalesBoardSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<HazardousInventoryVo> page = inventoryBoardService.getHazardousInventoryPage(dto);
        List<HazardousInventoryVo> list = page.getRecords();
        ExcelUtil.export(response,"危险库存", "危险库存数据", list, HazardousInventoryVo.class);
    }

    /**
     * 获取bom在途列表分页
     * @return
     */
    @PostMapping("/getInTransitBomPage")
    public Page<InTransitBomVo> getInTransitBomPage(@RequestBody SalesBoardSelectDto dto) {
        return inventoryBoardService.getInTransitBomPage(dto);
    }

    /**
     * bom在途列表导出
     * @return
     */
    @PostMapping("/inTransitBomExportExcel")
    public void inTransitBomExportExcel(HttpServletResponse response, @RequestBody SalesBoardSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<InTransitBomVo> page = inventoryBoardService.getInTransitBomPage(dto);
        List<InTransitBomVo> list = page.getRecords();
        ExcelUtil.export(response,"在途物料", "在途物料列表", list, InTransitBomVo.class);
    }

    /**
     * 获取库存周转率列表分页
     * @return
     */
    @PostMapping("/getTurnoverRatePage")
    public Page<InventoryTurnoverRateVo> getTurnoverRatePage(@RequestBody SalesBoardSelectDto dto) {
        return inventoryBoardService.getTurnoverRatePage(dto);
    }

    /**
     * 库存周转率列表导出
     * @return
     */
    @PostMapping("/turnoverRateExportExcel")
    public void turnoverRateExportExcel(HttpServletResponse response, @RequestBody SalesBoardSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<InventoryTurnoverRateVo> page = inventoryBoardService.getTurnoverRatePage(dto);
        List<InventoryTurnoverRateVo> list = page.getRecords();
        ExcelUtil.export(response,"库存周转率", "库存周转率列表", list, InventoryTurnoverRateVo.class);
    }

    /**
     * 获取采购入库列表分页
     * @return
     */
    @PostMapping("/getInOutStoragePage")
    public Page<InOutStorageBoardVo> getInOutStoragePage(@RequestBody InOutStorageBoardSelectDto dto) {
        return inventoryBoardService.getInOutStoragePage(dto);
    }

    /**
     * 出入库列表导出
     * @return
     */
    @PostMapping("/inOutStorageExportExcel")
    public void inOutStorageExportExcel(HttpServletResponse response, @RequestBody InOutStorageBoardSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<InOutStorageBoardVo> page = inventoryBoardService.getInOutStoragePage(dto);
        List<InOutStorageBoardVo> list = page.getRecords();
        ExcelUtil.export(response,"出入库数据", "导出数据", list, InOutStorageBoardVo.class);
    }
}
