package com.sd.business.service.inventory.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.inventory.dto.InventoryFinishedSelectDto;
import com.sd.business.entity.inventory.po.InventoryFinishedTemporary;
import com.sd.business.entity.inventory.vo.InventoryFinishedTemporaryVo;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.inventory.InventoryFinishedTemporaryMapper;
import com.sd.business.service.inventory.InventoryFinishedTemporaryService;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 成品暂存仓库 服务实现类
 * </p>
 *
 * @author
 * @since 2023-11-15
 */
@Service
public class InventoryFinishedTemporaryServiceImpl extends ServiceImpl<InventoryFinishedTemporaryMapper, InventoryFinishedTemporary> implements InventoryFinishedTemporaryService {

    @Override
    public Page<InventoryFinishedTemporaryVo> getPage(InventoryFinishedSelectDto dto) {
        IWrapper<InventoryFinishedTemporary> wrapper = getWrapper();
        wrapper.orderByDesc("i", InventoryFinishedTemporary::getId);
        wrapper.like("ss", SkuSpec::getCode, dto.getSkuSpecCode());
        wrapper.like("ss", SkuSpec::getName, dto.getSkuSpecName());
        wrapper.gt("i", InventoryFinishedTemporary::getQuantity, 0);
        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Override
    public void inOut(List<InventoryFinishedTemporary> list, boolean isIn) {
        if (ObjectUtil.isEmpty(list)) {
            return;
        }

        // 相同sku合并
        Map<Long, InventoryFinishedTemporary> map = list.stream().collect(Collectors.toMap(
                InventoryFinishedTemporary::getSkuSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                }));

        synchronized (this) {

            // 获取原库存数据
            Map<Long, InventoryFinishedTemporary> oldStorageMap = this.mapKEntity(InventoryFinishedTemporary::getSkuSpecId,
                    q -> q.in(InventoryFinishedTemporary::getSkuSpecId, map.keySet()));

            // 计算入库后库存数量
            for (InventoryFinishedTemporary item : map.values()) {
                InventoryFinishedTemporary inventoryFinishedTemporary = oldStorageMap.get(item.getSkuSpecId());
                if (inventoryFinishedTemporary == null) {
                    continue;
                }
                BigDecimal quantity;
                if (isIn) {
                    quantity = inventoryFinishedTemporary.getQuantity().add(item.getQuantity());
                } else {
                    quantity = inventoryFinishedTemporary.getQuantity().subtract(item.getQuantity());
                }
                item.setId(inventoryFinishedTemporary.getId());
                item.setQuantity(quantity);
            }

            // 更新数据库
            saveOrUpdateBatch(map.values());
        }
    }
}
