package com.sd.business.service.production.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.in.bo.InOutStorageBomBo;
import com.sd.business.entity.in.dto.InOutStorageDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.emums.OutDetailTypeEnum;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.inventory.po.InventoryFinishedOrder;
import com.sd.business.entity.order.enums.OrderClassifyEnum;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderPackageBom;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuBom;
import com.sd.business.entity.production.dto.StockPreparationDto;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.production.vo.*;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.statement.dto.StatementOfAccountDto;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.production.StockPreparationMapper;
import com.sd.business.service.bom.BomClassifyService;
import com.sd.business.service.bom.BomService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.inventory.InventoryFinishedOrderService;
import com.sd.business.service.inventory.InventoryFinishedService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.order.OrderPackageBomService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuBomService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.business.service.production.StockPreparationService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
public class StockPreparationServiceImpl implements StockPreparationService {

    @Resource
    private StockPreparationMapper stockPreparationMapper;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private ProductionWorkOrderService productionWorkOrderService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private BomService bomService;

    @Autowired
    private BomClassifyService bomClassifyService;

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Autowired
    private InventoryFinishedService inventoryFinishedService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Autowired
    private InventoryFinishedOrderService inventoryFinishedOrderService;

    @Override
    public List<UncompletedVo> uncompletedList(StockPreparationDto dto) {

        IWrapper<StockPreparationVo> wrapper = getWrapper(dto);
        List<StockPreparationVo> stockPreparationVoList = stockPreparationMapper.uncompletedList(wrapper);

        if (ObjectUtil.isEmpty(stockPreparationVoList)) {
            return Collections.emptyList();
        }

        // 查询出委外订单，委外订单不出库主材
        List<Long> orderIds = stockPreparationVoList.stream().map(StockPreparationVo::getOrderId).collect(Collectors.toList());
        List<Long> outsourceOrderIds = orderService.list(q -> q
                .in(BaseIdPo::getId, orderIds)
                .eq(OrderInfo::getType, 2)
                .select(BaseIdPo::getId))
                .stream().map(BaseIdPo::getId).collect(Collectors.toList());

        return stockPreparationVoList.stream()
                .filter(item -> outsourceOrderIds.isEmpty() || !outsourceOrderIds.contains(item.getOrderId()))
                .collect(Collectors.toMap(
                        StockPreparationVo::getBomSpecId,
                        item -> {
                            UncompletedVo uncompletedVo = new UncompletedVo();
                            uncompletedVo.setBomSpecId(item.getBomSpecId());
                            uncompletedVo.setBomSpecCode(item.getBomSpecCode());
                            uncompletedVo.setBomSpecName(item.getBomSpecName());
                            uncompletedVo.setTotalQuantity(item.getQuantity());

                            UncompletedVo.SkuInfo skuInfo = new UncompletedVo.SkuInfo();
                            skuInfo.setSkuSpecId(item.getSkuSpecId());
                            skuInfo.setSkuSpecCode(item.getSkuSpecCode());
                            skuInfo.setSkuSpecName(item.getSkuSpecName());
                            skuInfo.setQuantity(item.getQuantity());
                            skuInfo.setMachinedPanel(item.getMachinedPanel());
                            skuInfo.setArtworkName(item.getArtworkName());

                            List<UncompletedVo.SkuInfo> skuInfoList = new ArrayList<>();
                            skuInfoList.add(skuInfo);

                            uncompletedVo.setSkuInfoList(skuInfoList);

                            return uncompletedVo;
                        },
                        (v1, v2) -> {
                            v1.setTotalQuantity(v1.getTotalQuantity().add(v2.getTotalQuantity()));
                            UncompletedVo.SkuInfo skuInfo = v2.getSkuInfoList().get(0);
                            boolean flag = true;
                            for (UncompletedVo.SkuInfo temp : v1.getSkuInfoList()) {
                                if (Objects.equals(temp.getSkuSpecId(), skuInfo.getSkuSpecId())) {
                                    temp.setQuantity(temp.getQuantity().add(skuInfo.getQuantity()));
                                    flag = false;
                                    break;
                                }
                            }

                            if (flag) {
                                v1.getSkuInfoList().add(skuInfo);
                            }

                            return v1;
                        }
                ))
                .values()
                .stream()
                .sorted(Comparator.comparing(UncompletedVo::getBomSpecCode))
                .collect(Collectors.toList());
    }

    @Override
    public List<PrintUncompletedVo> printUncompletedList(StockPreparationDto dto) {

        IWrapper<StockPreparationVo> wrapper = getWrapper(dto);
        List<StockPreparationVo> stockPreparationVoList = stockPreparationMapper.uncompletedList(wrapper);

        if (ObjectUtil.isEmpty(stockPreparationVoList)) {
            return Collections.emptyList();
        }

        // 查询出委外订单，委外订单不出库主材
        List<Long> orderIds = stockPreparationVoList.stream().map(StockPreparationVo::getOrderId).collect(Collectors.toList());
        List<Long> outsourceOrderIds = orderService.list(q -> q
                        .in(BaseIdPo::getId, orderIds)
                        .eq(OrderInfo::getType, 2)
                        .select(BaseIdPo::getId))
                .stream().map(BaseIdPo::getId).collect(Collectors.toList());

        // 根据规格分组后再根据颜色分组
        Map<String, Map<String, List<StockPreparationVo>>> map = stockPreparationVoList.stream()
                .collect(
                        Collectors.groupingBy(
                                item -> item.getLength() + "*" + item.getWidth() + "*" + item.getHeight(),
                                Collectors.groupingBy(StockPreparationVo::getColour)
                        ));

        List<PrintUncompletedVo> list = new ArrayList<>();
        for (String spec : map.keySet()) {
            Map<String, List<StockPreparationVo>> specListMap = map.get(spec);
            List<PrintUncompletedVo.ColourGroupingInfo> groupingInfoList = new ArrayList<>();
            for (String colour : specListMap.keySet()) {
                PrintUncompletedVo.ColourGroupingInfo colourGroupingInfo = new PrintUncompletedVo.ColourGroupingInfo();
                colourGroupingInfo.setColour(colour);
                colourGroupingInfo.setUncompletedVoList(specListMap.get(colour).stream()
                        .filter(item -> outsourceOrderIds.isEmpty() || !outsourceOrderIds.contains(item.getOrderId()))
                        .collect(Collectors.toMap(
                                StockPreparationVo::getBomSpecId,
                                item -> {
                                    UncompletedVo uncompletedVo = new UncompletedVo();
                                    uncompletedVo.setBomSpecId(item.getBomSpecId());
                                    uncompletedVo.setBomSpecCode(item.getBomSpecCode());
                                    uncompletedVo.setBomSpecName(item.getBomSpecName());
                                    uncompletedVo.setTotalQuantity(item.getQuantity());

                                    UncompletedVo.SkuInfo skuInfo = new UncompletedVo.SkuInfo();
                                    skuInfo.setSkuSpecId(item.getSkuSpecId());
                                    skuInfo.setSkuSpecCode(item.getSkuSpecCode());
                                    skuInfo.setSkuSpecName(item.getSkuSpecName());
                                    skuInfo.setQuantity(item.getQuantity());
                                    skuInfo.setMachinedPanel(item.getMachinedPanel());
                                    skuInfo.setArtworkName(item.getArtworkName());

                                    List<UncompletedVo.SkuInfo> skuInfoList = new ArrayList<>();
                                    skuInfoList.add(skuInfo);

                                    uncompletedVo.setSkuInfoList(skuInfoList);

                                    return uncompletedVo;
                                },
                                (v1, v2) -> {
                                    v1.setTotalQuantity(v1.getTotalQuantity().add(v2.getTotalQuantity()));
                                    UncompletedVo.SkuInfo skuInfo = v2.getSkuInfoList().get(0);
                                    boolean flag = true;
                                    for (UncompletedVo.SkuInfo temp : v1.getSkuInfoList()) {
                                        if (Objects.equals(temp.getSkuSpecId(), skuInfo.getSkuSpecId())) {
                                            temp.setQuantity(temp.getQuantity().add(skuInfo.getQuantity()));
                                            flag = false;
                                            break;
                                        }
                                    }

                                    if (flag) {
                                        v1.getSkuInfoList().add(skuInfo);
                                    }

                                    return v1;
                                }
                        ))
                        .values()
                        .stream()
                        .sorted(Comparator.comparing(UncompletedVo::getBomSpecCode))
                        .collect(Collectors.toList()));
                groupingInfoList.add(colourGroupingInfo);
            }

            PrintUncompletedVo printUncompletedVo = new PrintUncompletedVo();
            printUncompletedVo.setSpec(spec);
            printUncompletedVo.setColourGroupingInfoList(groupingInfoList);
            list.add(printUncompletedVo);
        }
        return list;
    }

    @Override
    public Page<StockPreparationVo> completedPage(StockPreparationDto dto) {
        IWrapper<StockPreparationVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("oi", OrderInfo::getCode, dto.getOrderCode());
        wrapper.eq("oi", OrderInfo::getWlnCode, dto.getOrderWlnCode());
        wrapper.eq("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.eq("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.eq("d", Department::getId, dto.getDepartmentId());
        wrapper.eq("ss", SkuSpec::getCode, dto.getSkuSpecCode());
        wrapper.eq("ss", SkuSpec::getName, dto.getSkuSpecName());
        wrapper.eq("bs", BomSpec::getWidth, dto.getWidth());
        wrapper.eq("os", OrderSku::getStockPreparationStatus, StatusConstant.YES);
        wrapper.orderByDesc("os", OrderSku::getStockPreparationTime);
        wrapper.orderByAsc("os", OrderSku::getId);
        return stockPreparationMapper.completedPage(dto.getPage(), wrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public synchronized void submit(StockPreparationDto dto) {

        Long outDepartmentId = dto.getOutDepartmentId();
        Assert.notNull(outDepartmentId, "出库事业部id不能为空");

        List<StockPreparationVo> list = stockPreparationMapper.uncompletedList(getWrapper(dto));
        if (list.isEmpty()) {
            return;
        }

        Date stockPreparationTime = new Date();
        List<Long> orderIdList = list.stream().map(StockPreparationVo::getOrderId).distinct().collect(Collectors.toList());

        // 订单状态改为生产中
        orderService.update(q -> q
                .in(BaseIdPo::getId, orderIdList)
                .set(OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey())
                .set(OrderInfo::getOutDepartmentId, outDepartmentId)
                .set(OrderInfo::getLockStorage, StatusConstant.NO)
        );

        // 订单sku出库
        orderSkuService.update(q -> q
                .in(OrderSku::getOrderId, orderIdList)
                .set(OrderSku::getStockPreparationTime, stockPreparationTime)
                .set(OrderSku::getStockPreparationStatus, StatusConstant.YES));

        // 订单出库
        if (Objects.equals(dto.getOrderStockType(), 1)) {
            // 成品库出库
            orderFinishedDelivery(list, dto);
        } else {
            // 半成品出库
            orderDelivery(list, dto);
        }

        List<ProductionWorkOrder> productionWorkOrderList = new ArrayList<>();

        // 生产工单
        Map<Long, List<StockPreparationVo>> map = list.stream().collect(Collectors.groupingBy(StockPreparationVo::getOrderId));
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, map.keySet()));
        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);

        // 筛选掉不是主材的sku
        List<Long> bomSpecIds = orderSkuList.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIds);

        Map<Long, List<OrderSku>> orderSkuMap = orderSkuList.stream()
                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                .filter(item -> bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId().equals(1L))
                .collect(Collectors.groupingBy(OrderSku::getOrderId));

        map.forEach((orderId, stockPreparationVoList) -> {
            StockPreparationVo stockPreparationVo = stockPreparationVoList.get(0);
            String orderCode = stockPreparationVo.getOrderCode();
            String orderWlnCode = stockPreparationVo.getOrderWlnCode();
            addWorkOrder(orderId, ObjectUtil.defaultIfNull(orderWlnCode, orderCode),
                    orderSkuMap.getOrDefault(orderId, Collections.emptyList()), productionWorkOrderList);
        });

        if (!productionWorkOrderList.isEmpty()) {
            skuSpecService.attributeAssign(productionWorkOrderList, ProductionWorkOrder::getSkuSpecId,
                    (item, skuSpec) -> item.setMachinedPanel(skuSpec.getMachinedPanel()));
            productionWorkOrderService.saveBatch(productionWorkOrderList);
        }

        List<OrderInfo> orderInfoList = orderService.list(q -> q.in(BaseIdPo::getId, orderIdList));
        // 订单分类为无理由的订单，直接完成并生成对账单
        List<OrderInfo> statementOrderList = orderInfoList.stream().filter(item -> ObjectUtil.equals(item.getClassify(), OrderClassifyEnum.NO_REASON_ORDER.getKey())).collect(Collectors.toList());
        // 生成对账单
        if (ObjectUtil.isNotEmpty(statementOrderList)) {
            List<Long> orderIds = statementOrderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());

            orderService.update(q -> q
                    .in(BaseIdPo::getId, orderIds)
                    .set(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey())
                    .set(OrderInfo::getShippingTime, new Date()));

            Map<Long, List<OrderInfo>> statementMap = statementOrderList.stream().collect(Collectors.groupingBy(OrderInfo::getDepartmentId));
            for (Long departmentId : statementMap.keySet()) {
                List<Long> statementOrderIds = statementMap.get(departmentId).stream().map(BaseIdPo::getId).collect(Collectors.toList());
                StatementOfAccountDto statement = new StatementOfAccountDto();
                statement.setDepartmentId(departmentId);
                statement.setOrderIdList(statementOrderIds);
                statementOfAccountService.add(statement);
            }

            // 生产入库
            inventoryFinishedService.productionWarehousing(orderIds);
        }

        // 成品出库不需要生产裸垫
        if (Objects.equals(dto.getOrderStockType(), 0)) {
            // 把生产工单发送到mq
            productionWorkOrderService.sendMq(productionWorkOrderList);
        }
    }

    private void orderFinishedDelivery(List<StockPreparationVo> list, StockPreparationDto dto) {
        List<Long> orderIdList = list.stream().map(StockPreparationVo::getOrderId).distinct().collect(Collectors.toList());

        // 查询出委外订单，委外订单不出库主材
        List<OrderInfo> noReasonOrderList = orderService.list(q -> q
                .in(BaseIdPo::getId, orderIdList)
                .eq(OrderInfo::getClassify, OrderClassifyEnum.NO_REASON_ORDER.getKey())
                .select(BaseIdPo::getId));
        List<Long> noReasonOrderIds = noReasonOrderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());

        List<Long> orderPackageIds = new ArrayList<>(orderIdList);
        orderPackageIds.removeAll(noReasonOrderIds);

        if (!orderPackageIds.isEmpty()) {
            // 出库快递包装包材
            List<OrderPackageBom> orderPackageBomList = orderPackageBomService.list(q -> q.in(OrderPackageBom::getOrderId, orderPackageIds));
            if (!orderPackageBomList.isEmpty()) {
                // 快递包材
                List<InOutStorageBom> outStorageBomList = orderPackageBomList.stream()
                        .map(item -> {
                            InOutStorageBom inOutStorageBom = new InOutStorageBom();
                            inOutStorageBom.setBomSpecId(item.getBomSpecId());
                            inOutStorageBom.setQuantity(item.getQuantity());
                            return inOutStorageBom;
                        })
                        .collect(Collectors.toList());
                productionOut(WarehouseConstant.PACKAGING_MATERIAL, dto, outStorageBomList, StatusConstant.NO);
            }
        }
        // 成品出库
        inventoryFinishedService.noSourceSaleOutOfWarehouse(orderIdList);
    }

    @Override
    public List<PackageBomVo> getPackageBomList(List<Long> orderSkuIdList) {

        List<OrderSkuBom> list = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderSkuId, orderSkuIdList));

        List<PackageBomVo> result = new ArrayList<>(list.stream()
                .map(item -> PackageBomVo.builder().bomSpecId(item.getBomSpecId()).quantity(item.getQuantity()).build())
                .collect(Collectors.toMap(
                        PackageBomVo::getBomSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            return v1;
                        }
                )).values());

        bomSpecService.attributeAssign(result, PackageBomVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
            item.setBomId(bomSpec.getBomId());
        });

        bomService.attributeAssign(result, PackageBomVo::getBomId, (item, bom) -> {
            item.setBomClassifyId(bom.getBomClassifyId());
        });

        bomClassifyService.attributeAssign(result, PackageBomVo::getBomClassifyId, (item, bomClassify) -> {
            item.setBomClassifyName(bomClassify.getName());
        });

        return result;
    }

    @Override
    public List<OutBomVo> outBomList(StockPreparationDto dto) {
        Long outDepartmentId = dto.getOutDepartmentId();
        Assert.notNull(outDepartmentId, "出库事业部id不能为空");
        dto.setOrderStockType(0);

        // 订单sku列表
        IWrapper<StockPreparationVo> wrapper = getWrapper(dto);
        List<StockPreparationVo> stockPreparationVoList = stockPreparationMapper.uncompletedList(wrapper);

        if (stockPreparationVoList.isEmpty()) {
            return Collections.emptyList();
        }

        List<Long> orderIdList = stockPreparationVoList.stream().map(StockPreparationVo::getOrderId).distinct().collect(Collectors.toList());
        // 查询出委外订单，委外订单不出库主材
        List<OrderInfo> specialOrderList = orderService.list(q -> q
                .in(BaseIdPo::getId, orderIdList)
                .and(e -> e.eq(OrderInfo::getType, 2).or().eq(OrderInfo::getClassify, OrderClassifyEnum.NO_REASON_ORDER.getKey()))
                .select(BaseIdPo::getId, OrderInfo::getType));
        // 委外订单id集合
        List<Long> outsourceOrderIds = specialOrderList.stream()
                .filter(item -> Objects.equals(item.getType(), 2))
                .map(BaseIdPo::getId)
                .collect(Collectors.toList());
        // 无理由订单id集合
        List<Long> noReasonOrderIds = specialOrderList.stream()
                .filter(item -> !Objects.equals(item.getType(), 2))
                .map(BaseIdPo::getId)
                .collect(Collectors.toList());

        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderId, orderIdList));
        // 无理由订单不出库快递包材
        List<Long> orderPackageIds = new ArrayList<>(orderIdList);
        orderPackageIds.removeAll(noReasonOrderIds);
        List<OrderPackageBom> orderPackageBomList;
        if (orderPackageIds.isEmpty()) {
            orderPackageBomList = Collections.emptyList();
        } else {
            orderPackageBomList = orderPackageBomService.list(q -> q.in(OrderPackageBom::getOrderId, orderPackageIds));
        }
        Map<Long, OrderSku> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);

        Map<Long, OutBomVo> map = Stream.of(
                        // 主材
                        orderSkuList.stream()
                                .filter(item -> outsourceOrderIds.isEmpty() || !outsourceOrderIds.contains(item.getOrderId()))
                                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                                .map(item -> {
                                    OutBomVo outBomVo = new OutBomVo();
                                    outBomVo.setBomSpecId(item.getBomSpecId());
                                    outBomVo.setOutQuantity(item.getQuantity());
                                    return outBomVo;
                                }),

                        // 包材
                        orderSkuBomList.stream().map(item -> {
                            OrderSku orderSku = orderSkuMap.get(item.getOrderSkuId());
                            OutBomVo outBomVo = new OutBomVo();
                            outBomVo.setBomSpecId(item.getBomSpecId());
                            outBomVo.setOutQuantity(item.getQuantity().multiply(orderSku.getQuantity()));
                            return outBomVo;
                        }),

                        // 快递包材
                        orderPackageBomList.stream().map(item -> {
                            OutBomVo outBomVo = new OutBomVo();
                            outBomVo.setBomSpecId(item.getBomSpecId());
                            outBomVo.setOutQuantity(item.getQuantity());
                            return outBomVo;
                        })
                )
                .flatMap(stream -> stream)
                .collect(Collectors.toMap(
                        OutBomVo::getBomSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setOutQuantity(v1.getOutQuantity().add(v2.getOutQuantity()));
                            return v1;
                        })
                );

        // 合并sku主材和包材
        List<OutBomVo> outBomVoList = new ArrayList<>(map.values());

        // 区分半成品仓和包材仓库
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(map.keySet());

        for (OutBomVo outBomVo : outBomVoList) {
            Long bomSpecId = outBomVo.getBomSpecId();
            BomSpecBo bomSpecBo = bomSpecBoMap.get(bomSpecId);
            outBomVo.setBomSpecCode(bomSpecBo.getBomSpecCode());
            outBomVo.setBomSpecName(bomSpecBo.getBomSpecName());
            outBomVo.setClassifyId(bomSpecBo.getClassifyId());
            outBomVo.setClassifyName(bomSpecBo.getClassifyName());
            outBomVo.setClassifyParentId(bomSpecBo.getClassifyParentId());

            // 主材
            if (Objects.equals(bomSpecBo.getClassifyParentId(), 1L)) {
                outBomVo.setWarehouseId(WarehouseConstant.SEMI_FINISHED_PRODUCT);
                outBomVo.setWarehouseName("半成品仓");
            }
            // 包材
            else {
                outBomVo.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
                outBomVo.setWarehouseName("包材仓");
            }
        }

        // 主材库存
        Map<Long, BigDecimal> semiFinishedProductInventoryMap = Collections.emptyMap();
        List<Long> semiFinishedProductBomSpecIdList = outBomVoList.stream()
                .filter(item -> WarehouseConstant.SEMI_FINISHED_PRODUCT.equals(item.getWarehouseId()))
                .map(OutBomVo::getBomSpecId)
                .collect(Collectors.toList());
        if (!semiFinishedProductBomSpecIdList.isEmpty()) {
            List<Inventory> tempList = inventoryService.list(q -> q.eq(Inventory::getWarehouseId, WarehouseConstant.SEMI_FINISHED_PRODUCT)
                    .eq(Inventory::getDepartmentId, outDepartmentId)
                    .in(Inventory::getBomSpecId, semiFinishedProductBomSpecIdList));
            semiFinishedProductInventoryMap = tempList.stream().collect(Collectors.toMap(Inventory::getBomSpecId,
                    item -> item.getQuantity().add(item.getLockQuantity()), (v1, v2) -> v2));

        }

        // 包材库存
        Map<Long, BigDecimal> packagingMaterialInventoryMap = Collections.emptyMap();
        List<Long> packagingMaterialBomSpecIdList = outBomVoList.stream()
                .filter(item -> WarehouseConstant.PACKAGING_MATERIAL.equals(item.getWarehouseId()))
                .map(OutBomVo::getBomSpecId)
                .collect(Collectors.toList());
        if (!packagingMaterialBomSpecIdList.isEmpty()) {
            List<Inventory> tempList = inventoryService.list(q -> q
                    .eq(Inventory::getWarehouseId, WarehouseConstant.PACKAGING_MATERIAL)
                    .eq(Inventory::getDepartmentId, outDepartmentId)
                    .in(Inventory::getBomSpecId, packagingMaterialBomSpecIdList)
            );
            packagingMaterialInventoryMap = tempList.stream().collect(Collectors.toMap(Inventory::getBomSpecId,
                    item -> item.getQuantity().add(item.getLockQuantity()), (v1, v2) -> v2));
        }

        // 赋值库存
        for (OutBomVo outBomVo : outBomVoList) {
            if (WarehouseConstant.SEMI_FINISHED_PRODUCT.equals(outBomVo.getWarehouseId())) {
                outBomVo.setInventoryQuantity(semiFinishedProductInventoryMap.getOrDefault(outBomVo.getBomSpecId(), BigDecimal.ZERO));
            } else {
                outBomVo.setInventoryQuantity(packagingMaterialInventoryMap.getOrDefault(outBomVo.getBomSpecId(), BigDecimal.ZERO));
            }
        }

        return outBomVoList.stream()
                .sorted(Comparator.comparing(OutBomVo::getWarehouseId, Comparator.reverseOrder()).thenComparing(OutBomVo::getBomSpecCode))
                .collect(Collectors.toList());

    }

    @Override
    public List<OutSkuVo> outSkuList(StockPreparationDto dto) {
        Long outDepartmentId = dto.getOutDepartmentId();
        Assert.notNull(outDepartmentId, "出库事业部id不能为空");
        dto.setOrderStockType(1);

        // 订单sku列表
        IWrapper<StockPreparationVo> wrapper = getWrapper(dto);
        List<StockPreparationVo> stockPreparationVoList = stockPreparationMapper.uncompletedList(wrapper);

        if (stockPreparationVoList.isEmpty()) {
            return Collections.emptyList();
        }

        List<Long> orderIdList = stockPreparationVoList.stream().map(StockPreparationVo::getOrderId).distinct().collect(Collectors.toList());
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));

        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);

        Map<Long, OutSkuVo> map = orderSkuList.stream()
                .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                .map(item -> {
                    OutSkuVo outSkuVo = new OutSkuVo();
                    outSkuVo.setSkuSpecId(item.getSkuSpecId());
                    outSkuVo.setOutQuantity(item.getQuantity());
                    return outSkuVo;
                })
                .collect(Collectors.toMap(
                        OutSkuVo::getSkuSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setOutQuantity(v1.getOutQuantity().add(v2.getOutQuantity()));
                            return v1;
                        })
                );

        List<OutSkuVo> outBomVoList = new ArrayList<>(map.values());

        Map<Long, SkuSpec> skuSpecMap = skuSpecService.byIdsToMap(map.keySet());
        // 查询成品库存
        List<InventoryFinishedOrder> list = inventoryFinishedOrderService.list(
                q -> q.in(InventoryFinishedOrder::getSkuSpecId, map.keySet())
                        .eq(InventoryFinishedOrder::getStatus, 1)
                        .isNull(InventoryFinishedOrder::getOrderInfoId));
        Map<Long, BigDecimal> inventoryMap = list.stream()
                .collect(Collectors.toMap(
                        InventoryFinishedOrder::getSkuSpecId,
                        InventoryFinishedOrder::getExistingQuantity,
                        BigDecimal::add)
                );

        // 赋值
        for (OutSkuVo outSkuVo : outBomVoList) {
            Long skuSpecId = outSkuVo.getSkuSpecId();
            SkuSpec skuSpec = skuSpecMap.get(skuSpecId);
            outSkuVo.setSkuSpecCode(skuSpec.getCode());
            outSkuVo.setSkuSpecName(skuSpec.getName());
            outSkuVo.setWarehouseId(WarehouseConstant.FINISHED_PRODUCT);
            outSkuVo.setWarehouseName("成品仓");
            outSkuVo.setInventoryQuantity(inventoryMap.getOrDefault(skuSpecId, BigDecimal.ZERO));
        }

        // 获取快递包装包材，筛选委外订单不出库主材
        List<OrderInfo> noReasonOrderList = orderService.list(q -> q
                .in(BaseIdPo::getId, orderIdList)
                .eq(OrderInfo::getClassify, OrderClassifyEnum.NO_REASON_ORDER.getKey())
                .select(BaseIdPo::getId));
        List<Long> noReasonOrderIds = noReasonOrderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        orderIdList.removeAll(noReasonOrderIds);

        if (orderIdList.isEmpty()) {
            return outBomVoList;
        }

        List<OrderPackageBom> orderPackageBomList = orderPackageBomService.list(q -> q.in(OrderPackageBom::getOrderId, orderIdList));
        if (orderPackageBomList.isEmpty()) {
            return outBomVoList;
        }
        // 快递包材
        Map<Long, OutSkuVo> skuPackageBomVoMap = orderPackageBomList.stream()
                .map(item -> {
                    OutSkuVo outSkuVo = new OutSkuVo();
                    outSkuVo.setSkuSpecId(item.getBomSpecId());
                    outSkuVo.setOutQuantity(item.getQuantity());
                    return outSkuVo;
                })
                .collect(Collectors.toMap(
                        OutSkuVo::getSkuSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setOutQuantity(v1.getOutQuantity().add(v2.getOutQuantity()));
                            return v1;
                        })
                );
        List<OutSkuVo> skuPackageBomVoList = new ArrayList<>(skuPackageBomVoMap.values());
        Map<Long, BomSpec> bomSpecMap = bomSpecService.mapKEntity(BaseIdPo::getId, q -> q.in(BaseIdPo::getId, skuPackageBomVoMap.keySet()));
        // 查询库存
        Map<Long, BigDecimal> skuPackageBomInventoryMap = inventoryService.mapKV(Inventory::getBomSpecId, Inventory::getQuantity, q -> q
                .eq(Inventory::getWarehouseId, WarehouseConstant.PACKAGING_MATERIAL)
                .eq(Inventory::getDepartmentId, outDepartmentId)
                .in(Inventory::getBomSpecId, skuPackageBomVoMap.keySet()));
        for (OutSkuVo outSkuVo : skuPackageBomVoList) {
            Long bomSpecId = outSkuVo.getSkuSpecId();
            BomSpec bomSpec = bomSpecMap.get(bomSpecId);
            outSkuVo.setSkuSpecCode(bomSpec.getCode());
            outSkuVo.setSkuSpecName(bomSpec.getName());
            outSkuVo.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
            outSkuVo.setWarehouseName("包材仓");
            outSkuVo.setInventoryQuantity(skuPackageBomInventoryMap.getOrDefault(bomSpecId, BigDecimal.ZERO));
        }
        // 添加快递包材
        outBomVoList.addAll(skuPackageBomVoList);

        return outBomVoList;
    }

    @Override
    public UncompletedStatisticsVo uncompletedListStatistics() {
        IWrapper<StockPreparationVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.STOCK_PREPARATION.getKey());
        wrapper.eq("oi", OrderInfo::getExceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString());
        wrapper.eq("os", OrderSku::getStockPreparationStatus, StatusConstant.NO);
        wrapper.and(q -> q
                .ge("oi", OrderInfo::getWlnStatus, 2)
                .or().eq("oi", OrderInfo::getSource, 1));
        wrapper.and(q -> q
                .and(r -> r.isNull("oi.flow_id").isNull("oi.flow_status"))
                .or(r -> r.eq("oi", OrderInfo::getFlowStatus, FlowStatusEnum.PASS.getKey())));
        // 不计算委外订单主材数量
        wrapper.and(q -> q.isNull("oi.type").or().ne("oi", OrderInfo::getType, 2));

        return stockPreparationMapper.uncompletedListStatistics(wrapper);
    }

    /**
     * 获取wrapper
     */
    private IWrapper<StockPreparationVo> getWrapper(StockPreparationDto dto) {
        Assert.notNull(dto.getDepartmentId(), "事业部id不能为空");

        IWrapper<StockPreparationVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.STOCK_PREPARATION.getKey());
        wrapper.eq("oi", OrderInfo::getExceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString());
        wrapper.eq("os", OrderSku::getStockPreparationStatus, StatusConstant.NO);
        wrapper.eq("oi", OrderInfo::getDepartmentId, dto.getDepartmentId());
        wrapper.eq("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.eq("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.eq("ss", SkuSpec::getCode, dto.getSkuSpecCode());
        wrapper.eq("ss", SkuSpec::getName, dto.getSkuSpecName());
        wrapper.and(q -> q.ge("oi", OrderInfo::getWlnStatus, 2).or().eq("oi", OrderInfo::getSource, 1));

        wrapper.and(q -> q
                .and(r -> r.isNull("oi.flow_id").isNull("oi.flow_status"))
                .or(r -> r.eq("oi", OrderInfo::getFlowStatus, FlowStatusEnum.PASS.getKey())));

        wrapper.eq("oi", OrderInfo::getStockType, dto.getOrderStockType());

        return wrapper;
    }

    /**
     * 添加 生产任务、工单
     */
    private void addWorkOrder(Long id, String code, List<OrderSku> list, List<ProductionWorkOrder> productionWorkOrderList) {

        int num = 1;

        for (OrderSku orderSku : list) {
            for (int i = 0; i < orderSku.getQuantity().intValue(); i++) {
                ProductionWorkOrder productionWorkOrder = new ProductionWorkOrder();
                productionWorkOrder.setType(1);
                productionWorkOrder.setCode(code + String.format("-%05d", num));
                productionWorkOrder.setOrderId(id);
                productionWorkOrder.setOrderSkuId(orderSku.getId());
                productionWorkOrder.setSkuSpecId(orderSku.getSkuSpecId());
                productionWorkOrder.setBomSpecId(orderSku.getBomSpecId());
                productionWorkOrder.setPrintType(orderSku.getPrintType());
                productionWorkOrder.setStatus(0);
                productionWorkOrderList.add(productionWorkOrder);
                num++;
            }
        }

    }

    /**
     * 订单出库
     */
    private void orderDelivery(List<StockPreparationVo> stockPreparationVoList, StockPreparationDto dto) {

        Map<Long, Integer> orderIdLockStorageMap = stockPreparationVoList.stream().collect(Collectors.toMap(
                StockPreparationVo::getOrderId,
                item -> ObjectUtil.defaultIfNull(item.getLockStorage(), StatusConstant.NO),
                (v1, v2) -> v1)
        );
        Set<Long> orderIdList = orderIdLockStorageMap.keySet();
        // 查询出委外订单，委外订单不出库主材
        List<OrderInfo> specialOrderList = orderService.list(q -> q
                .in(BaseIdPo::getId, orderIdList)
                .and(e -> e.eq(OrderInfo::getType, 2).or().eq(OrderInfo::getClassify, OrderClassifyEnum.NO_REASON_ORDER.getKey()))
                .select(BaseIdPo::getId, OrderInfo::getType));
        // 委外订单id集合
        List<Long> outsourceOrderIds = specialOrderList.stream()
                .filter(item -> Objects.equals(item.getType(), 2))
                .map(BaseIdPo::getId)
                .collect(Collectors.toList());
        // 无理由订单id集合
        List<Long> noReasonOrderIds = specialOrderList.stream()
                .filter(item -> !Objects.equals(item.getType(), 2))
                .map(BaseIdPo::getId)
                .collect(Collectors.toList());

        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderId, orderIdList));
        Map<Long, OrderSku> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
        // 无理由订单不出库快递包材
        List<Long> orderPackageIds = new ArrayList<>(orderIdList);
        orderPackageIds.removeAll(noReasonOrderIds);
        List<OrderPackageBom> orderPackageBomList;
        if (orderPackageIds.isEmpty()) {
            orderPackageBomList = Collections.emptyList();
        } else {
            orderPackageBomList = orderPackageBomService.list(q -> q.in(OrderPackageBom::getOrderId, orderPackageIds));
        }
        // 筛选掉赠品sku
        List<Long> skuSpecIds = orderSkuList.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Set<Long> giftSkuSpecIds = skuSpecService.getGiftIdListByIdList(skuSpecIds);

        List<InOutStorageBomBo> inOutStorageBomBoList = Stream.of(

                // 主材
                orderSkuList.stream()
                        .filter(item -> outsourceOrderIds.isEmpty() || !outsourceOrderIds.contains(item.getOrderId()))
                        .filter(item -> giftSkuSpecIds.isEmpty() || !giftSkuSpecIds.contains(item.getSkuSpecId()))
                        .map(item -> {
                            InOutStorageBomBo inOutStorageBom = new InOutStorageBomBo();
                            inOutStorageBom.setBomSpecId(item.getBomSpecId());
                            inOutStorageBom.setQuantity(item.getQuantity());
                            inOutStorageBom.setLockStorage(orderIdLockStorageMap.get(item.getOrderId()));
                            return inOutStorageBom;
                        }),

                // 包材
                orderSkuBomList.stream().map(item -> {
                    InOutStorageBomBo inOutStorageBom = new InOutStorageBomBo();
                    inOutStorageBom.setBomSpecId(item.getBomSpecId());
                    inOutStorageBom.setQuantity(item.getQuantity().multiply(orderSkuMap.get(item.getOrderSkuId()).getQuantity()));
                    return inOutStorageBom;
                }),

                // 快递包材
                orderPackageBomList.stream().map(item -> {
                    InOutStorageBomBo inOutStorageBom = new InOutStorageBomBo();
                    inOutStorageBom.setBomSpecId(item.getBomSpecId());
                    inOutStorageBom.setQuantity(item.getQuantity());
                    return inOutStorageBom;
                })

        ).flatMap(item -> item).collect(Collectors.toList());

        List<Long> bomSpecIdList = inOutStorageBomBoList.stream().map(InOutStorageBom::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBo = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);

        // 主材从半成品仓出库(从冻结库存出库)
        List<InOutStorageBom> lockSemiFinishedProductInOutStorageBomList = inOutStorageBomBoList.stream()
                .filter(item -> ObjectUtil.equal(bomSpecBo.get(item.getBomSpecId()).getClassifyParentId(), 1L))
                .filter(item -> ObjectUtil.equal(item.getLockStorage(), StatusConstant.YES))
                .collect(Collectors.toList());
        if (!lockSemiFinishedProductInOutStorageBomList.isEmpty()) {
            productionOut(WarehouseConstant.SEMI_FINISHED_PRODUCT, dto, lockSemiFinishedProductInOutStorageBomList, StatusConstant.YES);
        }

        // 主材从半成品仓出库(从非冻结库存出库)
        List<InOutStorageBom> semiFinishedProductInOutStorageBomList = inOutStorageBomBoList.stream()
                .filter(item -> ObjectUtil.equal(bomSpecBo.get(item.getBomSpecId()).getClassifyParentId(), 1L))
                .filter(item -> ObjectUtil.notEqual(item.getLockStorage(), StatusConstant.YES))
                .collect(Collectors.toList());
        if (!semiFinishedProductInOutStorageBomList.isEmpty()) {
            productionOut(WarehouseConstant.SEMI_FINISHED_PRODUCT, dto, semiFinishedProductInOutStorageBomList, StatusConstant.NO);
        }

        // 包材从包材仓出库
        List<InOutStorageBom> packagingMaterialInOutStorageBomList = inOutStorageBomBoList.stream()
                .filter(item -> ObjectUtil.notEqual(bomSpecBo.get(item.getBomSpecId()).getClassifyParentId(), 1L))
                .collect(Collectors.toList());
        if (!packagingMaterialInOutStorageBomList.isEmpty()) {
            productionOut(WarehouseConstant.PACKAGING_MATERIAL, dto, packagingMaterialInOutStorageBomList, StatusConstant.NO);
        }

    }

    /**
     * 生产出库
     *
     * @param warehouseId         出库id
     * @param dto                 入参
     * @param inOutStorageBomList 出库bom规格明细
     */
    private void productionOut(Long warehouseId, StockPreparationDto dto, List<InOutStorageBom> inOutStorageBomList, Integer lockStorage) {

        // 合并相同bom规格出库数量
        List<InOutStorageBom> list = new ArrayList<>(inOutStorageBomList.stream().collect(Collectors.toMap(
                InOutStorageBom::getBomSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                })).values());

        InOutStorageDto inOutStorageDto = new InOutStorageDto();
        inOutStorageDto.setType(InOutTypeEnum.OUT.getKey());
        inOutStorageDto.setDetailType(OutDetailTypeEnum.PRODUCTION.getKey());
        inOutStorageDto.setWarehouseId(warehouseId);
        inOutStorageDto.setDepartmentId(dto.getOutDepartmentId());
        inOutStorageDto.setApplicant(dto.getApplicant());
        inOutStorageDto.setRemark(dto.getRemark());
        inOutStorageDto.setInOutStorageBomList(list);
        inOutStorageDto.setLockStorage(lockStorage);
        inOutStorageService.add(inOutStorageDto);
    }

    /**
     * 把工单发送到mq
     */
    private void sendWorkOrderMq(List<ProductionWorkOrder> productionWorkOrderList) {

        if (productionWorkOrderList.isEmpty()) {
            return;
        }

//        Map<Long, ProductionTask> taskMap = productionTaskList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
//
//        productionWorkOrderList.stream().map(item -> {
//            Long productionTaskId = item.getProductionTaskId();
//            ProductionTask productionTask = taskMap.get(productionTaskId);
//
//
//            WorkOrderDetail workOrderDetail = new WorkOrderDetail();
//            workOrderDetail.setWorkOrderId(item.getId());
//
//
//
//        })

    }

}
