package com.sd.wln.service.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.fjhx.tenant.entity.dict.po.DictCommonData;
import com.fjhx.tenant.service.dict.DictCommonDataService;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.framework.mybatis.holder.LogicHolder;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.service.order.OrderService;
import com.sd.wln.context.OrderContext;
import com.sd.wln.entity.ResynchronizationDto;
import com.sd.wln.service.OrderHandleService;
import com.sd.wln.service.WlnOrderService;
import com.sd.wln.util.WlnUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
public class OrderHandleServiceImpl implements OrderHandleService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private WlnOrderService wlnOrderService;

    @Autowired
    private DictCommonDataService dictCommonDataService;

    @DSTransactional
    @Override
    public synchronized void resynchronization(ResynchronizationDto dto) {
        JSONObject wlnOrder;
        try {
            wlnOrder = WlnUtil.getOrderByWlnCode(dto.getWlnCode());
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            log.error("订单同步失败", e);
            throw new ServiceException("订单同步失败");
        }

        LogicHolder.setLogicHolder(true);
        String uid = wlnOrder.getString("uid");
        OrderInfo orderInfo = orderService.getOne(q -> q.eq(OrderInfo::getWlnUid, uid));
        if (orderInfo != null) {
            if (Objects.equals(orderInfo.getDelFlag(), StatusConstant.NOT_DELETED)) {
                throw new ServiceException("订单未被删除，无法重新同步");
            }
            orderService.delete(orderInfo.getId());
        }
        LogicHolder.clear();

        String storageCode = wlnOrder.getString("storage_code");
        OrderContext context = new OrderContext(storageCode, Collections.singletonList(wlnOrder));
        OrderInfo order = wlnOrderService.createOrder(context, wlnOrder);
        wlnOrderService.addOrder(context, wlnOrder, order);
        wlnOrderService.saveOrUpdateOrder(context);
    }

    @Override
    public void bathSyncOrder() {
        List<OrderInfo> list = orderService.list(q -> q.in(OrderInfo::getWlnStatus, 0, 1, 14));
        Map<String, OrderInfo> orderInfoMap = list.stream().collect(Collectors.toMap(OrderInfo::getWlnCode, Function.identity()));
        Map<String, List<OrderInfo>> map = list.stream().collect(Collectors.groupingBy(OrderInfo::getWlnStorageCode));

        List<String> storageCodeList = getStorageCodeList();
        if (ObjectUtil.isEmpty(storageCodeList)) {
            log.error("订单手动同步失败，仓库字典:warehouse_code 为空");
            return;
        }

        for (String warehouseCode : map.keySet()) {
            List<JSONObject> wlnOrderList = new ArrayList<>();

            // list切分，万里牛最多只能查询200条
            List<List<OrderInfo>> split = ListUtil.split(map.get(warehouseCode), 200);
            try {
                for (List<OrderInfo> item : split) {
                    String wlnCodeListStr = item.stream().map(OrderInfo::getWlnCode).collect(Collectors.joining(","));
                    wlnOrderList.addAll(WlnUtil.getOrderListByWlnCodeList(wlnCodeListStr));
                }
            } catch (Exception e) {
                log.error("订单手动同步失败", e);
                throw new ServiceException("订单同步失败:" + e.getMessage());
            }

            if (wlnOrderList.isEmpty()) {
                continue;
            }

            // 初始化订单上下文
            OrderContext context = new OrderContext(warehouseCode, wlnOrderList);

            for (JSONObject wlnOrder : wlnOrderList) {
                // 非指定事业部订单不同步
                String storageCode = wlnOrder.getString("storage_code");
                if (!storageCodeList.contains(storageCode)) {
                    continue;
                }

                OrderInfo orderInfo = orderInfoMap.get(wlnOrder.getString("trade_no"));
                if (Objects.equals(wlnOrder.getLong("modify_time"), orderInfo.getWlnModifyTimestamp())) {
                    continue;
                }
                wlnOrderService.updateOrder(context, wlnOrder, orderInfo);
            }

            wlnOrderService.save(context);
        }

    }

    private List<String> getStorageCodeList() {
        List<DictCommonData> warehouseCodeList = dictCommonDataService.list(
                q -> q.eq(DictCommonData::getDictCode, "warehouse_code"));
        if (warehouseCodeList.isEmpty()) {
            return Collections.emptyList();
        }
        return warehouseCodeList.stream().map(DictCommonData::getDictValue).collect(Collectors.toList());
    }

}
